@@ -924,51 +924,28 @@ class CTypesBackend(object):
         CTypesFunctionPtr._fix_class()
         return CTypesFunctionPtr
 
-    def new_enum_type(self, name, enumerators, enumvalues):
+    def new_enum_type(self, name, enumerators, enumvalues, CTypesInt):
         assert isinstance(name, str)
-        mapping = dict(zip(enumerators, enumvalues))
         reverse_mapping = dict(zip(reversed(enumvalues),
                                    reversed(enumerators)))
-        CTypesInt = self.ffi._get_cached_btype(model.PrimitiveType('int'))
-        #
-        def forward_map(source):
-            if not isinstance(source, str):
-                return source
-            try:
-                return mapping[source]
-            except KeyError:
-                if source.startswith('#'):
-                    try:
-                        return int(source[1:])
-                    except ValueError:
-                        pass
-            raise ValueError("%r is not an enumerator for %r" % (
-                source, CTypesEnum))
         #
         class CTypesEnum(CTypesInt):
             __slots__ = []
             _reftypename = 'enum %s &' % name
 
+            def _get_own_repr(self):
+                value = self._value
+                try:
+                    return '%d: %s' % (value, reverse_mapping[value])
+                except KeyError:
+                    return str(value)
+
             def _to_string(self, maxlen):
-                return str(CTypesEnum._from_ctypes(self._value))
-
-            @classmethod
-            def _cast_from(cls, source):
-                source = forward_map(source)
-                return super(CTypesEnum, cls)._cast_from(source)
-
-            @staticmethod
-            def _to_ctypes(x):
-                x = forward_map(x)
-                return CTypesInt._to_ctypes(x)
-
-            @staticmethod
-            def _from_ctypes(value):
-                value = CTypesInt._from_ctypes(value)
+                value = self._value
                 try:
                     return reverse_mapping[value]
                 except KeyError:
-                    return '#%s' % value
+                    return str(value)
         #
         CTypesEnum._fix_class()
         return CTypesEnum
