@@ -1,6 +1,10 @@
-import py, os
+import py, os, sys
 import cffi, _cffi_backend
 
+def setup_module(mod):
+    if '_cffi_backend' in sys.builtin_module_names:
+        py.test.skip("this is embedded version")
+
 BACKEND_VERSIONS = {
     '0.4.2': '0.4',     # did not change
     }
@@ -25,8 +29,7 @@ def test_doc_version_file():
     v = cffi.__version__
     p = os.path.join(parent, 'doc', 'source', 'index.rst')
     content = open(p).read()
-    if ("cffi/cffi-%s.tar.gz" % v) not in content:
-        py.test.skip("XXX fix the file referenced by the doc!")
+    assert ("cffi/cffi-%s.tar.gz" % v) in content
 
 def test_setup_version():
     parent = os.path.dirname(os.path.dirname(__file__))
@@ -35,3 +38,10 @@ def test_setup_version():
     #
     v = cffi.__version__
     assert ("version='%s'" % v) in content
+
+def test_c_version():
+    parent = os.path.dirname(os.path.dirname(__file__))
+    v = cffi.__version__
+    p = os.path.join(parent, 'c', 'test_c.py')
+    content = open(p).read()
+    assert ('assert __version__ == "%s"' % v) in content
